//: Playground - noun: a place where people can play

import UIKit

/////////////////////////////////////////
// code 3.81, 3.82
/////////////////////////////////////////
protocol AlphaProtocol {
    func printStr()
}
protocol BetaProtocol {}
class SomeClass {}

class C:SomeClass, AlphaProtocol {
    func printStr() {
        print("I'm class")
    }
}
struct S:AlphaProtocol, BetaProtocol {
    func printStr() {
        Swift.print("I'm struct")
    }
}
enum E:AlphaProtocol {
    func printStr() {
        Swift.print("I'm enum")
    }
}

/////////////////////////////////////////
// code 3.83
/////////////////////////////////////////
protocol InitializeProtocol {
    init(str:String)
}
class C2:InitializeProtocol {
    required init(str: String) {
        // implementation
    }
}

/////////////////////////////////////////
// code 3.84
/////////////////////////////////////////
protocol AlphaProtocol2 {
    var printString:String {get set}
    func printStr()
}

/////////////////////////////////////////
// code 3.85,3.86
/////////////////////////////////////////
protocol ClassFuncProtocol {
    static func myType() -> String
}
class C3:ClassFuncProtocol {
    class func myType() -> String {
        return "Class"
    }
}
struct S3:ClassFuncProtocol {
    static func myType() -> String {
        return "Struct"
    }
}
enum E3:ClassFuncProtocol {
    static func myType() -> String {
        return "Enum"
    }
}

/////////////////////////////////////////
// code 3.87
/////////////////////////////////////////
protocol InclementProtocol {
    var i:Int {get}
    mutating func inclement()
}
class C4: InclementProtocol {
    var i = 0
    func inclement() {
        i++
    }
}
struct S4: InclementProtocol {
    var i = 0
    mutating func inclement() {
        i++
    }
}

/////////////////////////////////////////
// code 3.91
/////////////////////////////////////////
protocol SomeProtocol {}
protocol Some2Protocol {}
class SomeProtocolImpl: SomeProtocol, Some2Protocol {}
var a:SomeProtocol = SomeProtocolImpl()
var collection:[SomeProtocol] = []
func someMethod(val:SomeProtocol) {}
var c5:protocol<SomeProtocol, Some2Protocol> = SomeProtocolImpl()

/////////////////////////////////////////
// code 3.93
/////////////////////////////////////////
class SomeType {}
extension SomeType {
    func appendMethod() {
        // implementation
    }
}
/////////////////////////////////////////
// code 3.95
/////////////////////////////////////////
struct Rect {
    var x:Double, y:Double, width:Double, height:Double
}
extension Rect {
    var top:Double {
        get {
            return y
        }
        set {
            y = newValue
        }
    }
    var bottom:Double {
        get {
            return y + height
        }
        set {
            y = newValue - height
        }
    }
}
extension Rect {
    static var zeroRect:Rect {
        get {
            return Rect(x: 0, y: 0, width: 0, height: 0)
        }
    }
}
var rect = Rect.zeroRect
rect.height = 100
print(rect.bottom)
rect.bottom = 200
print(rect.top)

/////////////////////////////////////////
// code 3.96
/////////////////////////////////////////
extension Rect {
    init(x:Double, y:Double, squareSideLength sideLength:Double) {
        self.init(x:x, y:y, width:sideLength, height:sideLength)
    }
}
var square = Rect(x: 0, y: 0, squareSideLength: 200)

/////////////////////////////////////////
// code 3.97
/////////////////////////////////////////
extension Rect {
    func area() -> Double {
        return width * height
    }
    mutating func move(x:Double, y:Double) {
        self.x += x
        self.y += y
    }
}

/////////////////////////////////////////
// code 3.98
/////////////////////////////////////////
let str = "Hello Swift"
extension String {
    subscript(index:Int) -> String? {
        let targetIndex = startIndex.advancedBy(index)
        guard targetIndex < endIndex else {
            return nil }
        return String(self[targetIndex])
    }
}
print(str[6])

/////////////////////////////////////////
// code 3.99
/////////////////////////////////////////
extension Rect {
    enum Kind {
        case NoArea, Squere, Portrait, Landscape
    }
    var kind:Kind {
        get {
            if self.area() == 0 {
                return .NoArea
            }
            if self.width == self.height {
                return .Squere
            }
            if self.width < self.height {
                return .Portrait
            }
            return .Landscape
        }
    }
}
var rect2 = Rect(x: 0, y: 0, width: 100, height: 200)
rect2.kind // Rect.kind.Potrait


/////////////////////////////////////////
// code 3.101, 3.102
/////////////////////////////////////////
class A: CustomStringConvertible {
    var description = "I'm A."
}

print(A())

extension A {
    func p() {
        print(self)
    }
}
A().p()

/////////////////////////////////////////
// code 3.103, 3.104
/////////////////////////////////////////
struct B: CustomStringConvertible {
    var description = "I'm B."
}
print(B())

extension B {
    func p() {
        print(self)
    }
}
B().p()

/////////////////////////////////////////
// code 3.105, 3.106
/////////////////////////////////////////
extension CustomStringConvertible {
    func p() {
        print(self)
    }
}

class A2: CustomStringConvertible {
    var description = "I'm A."
}
struct B2: CustomStringConvertible {
    var description = "I'm B."
}
A().p()
B().p()

/////////////////////////////////////////
// code 3.107, 3.108, 3.109
/////////////////////////////////////////
protocol MyProtocol {
    func greeting()
}
extension MyProtocol {
    func greeting() {
        print("hello!")
    }
}

class MyObject: MyProtocol {}
MyObject().greeting() // hello!


/////////////////////////////////////////
// code 3.110
/////////////////////////////////////////
@objc protocol MyObjCProtocol {
    optional func greeting()
}
class MyObjCObject: NSObject, MyObjCProtocol {}
let obj: MyObjCProtocol = MyObjCObject()
obj.greeting?()
MyObject().greeting()


/////////////////////////////////////////
// code 3.111
/////////////////////////////////////////
class MyObject2: MyProtocol {
    func greeting() {
        // super.greeting() はできない
        print(" こんにちは! ") }
}

/////////////////////////////////////////
// code 3.112
/////////////////////////////////////////
extension MyProtocol where Self: UIViewController {
    func greeting() {
        print("hello! I'm UIViewController subclass.")
    }
}
class MyViewController: UIViewController, MyProtocol {}
MyViewController().greeting() // hello! I'm UIViewController subclass.