//: Playground - noun: a place where people can play

import UIKit

// Playground - noun: a place where people can play

import UIKit

/////////////////////////////////////////
//
// 3-7-1 パターンマッチング
//
/////////////////////////////////////////

// 数値・範囲マッチング
var dbl = 1.5
switch dbl {
case 1.4:
    print("1.4")
case 1.5:
    print("1.5")
case 1.6:
    print("1.6")
default:
    break
}

switch dbl {
case 0..<1:
    print("0 〜 1")
case 1..<2:
    print("1 〜 2")
case 2..<3:
    print("2 〜 3")
default:
    break
}

// 文字列マッチング
var month = "August"
switch month {
case "April", "May", "June":
    print("1Q")
case "July", "August", "September":
    print("2Q")
case "October", "November", "December":
    print("3Q")
case "Juanuary", "February", "March":
    print("4Q")
default:
    break
}

month = "September"
switch month {
case _ where month.hasSuffix("ary"):
    print("1月 2月")
case _ where month.hasSuffix("ber"):
    print("9月 10月 11月 12月")
default:
    break
}

// 列挙体マッチング
enum Month {
    case January, February, March, April, May, June,
    July, August, September, October, November, December
}

var month2 = Month.December

switch month2 {
case .March, .April, .May:
    print("Spring")
case .June, .July, .August:
    print("Summer")
case .September, .October, .November:
    print("Autumn")
case .December, .January, .February:
    print("Winter")
}

// 関連値マッチング
enum Status {
    case Fine
    case Error(code: Int)
}

var status = Status.Error(code: 2)
switch status {
case .Fine:
    print("status is fine")
case .Error(let code) where code < 4:
    print("error")
case .Error:
    print("fatal error")
}

// 型マッチング
class AViewController: UIViewController {
}

class BViewController: UIViewController {
}

class ViewController: UIViewController {
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        switch segue.destinationViewController {
        case is AViewController:
            let vc = segue.destinationViewController as! AViewController
            // task for AViewController
        case is BViewController:
            let vc = segue.destinationViewController as! BViewController
            // task for BViewController
        default:
            break
        }
    }
}

// as演算子によるマッチング
class ViewController2: UIViewController {
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        switch segue.destinationViewController {
        case let vc as AViewController:
            // task for AViewController
            print(vc)
        case let vc as BViewController:
            // task for BViewController
            print(vc)
        default:
            break
        }
    }
}

/////////////////////////////////////////
//
// 3-7-2 switchのパターンマッチングの正体
//
/////////////////////////////////////////

// Rect型とPoint型のパターンマッチ
struct Rect {
    var top: Double
    var left: Double
    var width: Double
    var height: Double
}

struct Point {
    var x: Double
    var y: Double
}

func ~= (lhs:Rect, rhs:Point) -> Bool {
    return (lhs.left...lhs.left + lhs.width ~= rhs.x) &&
        (lhs.top...lhs.top + lhs.height ~= rhs.y)
}

var p = Point(x: 100, y: 100)

switch p {
case Rect(top: 0, left: 0, width: 200, height: 200):
    print("in x: positive, y:positive rect")
case Rect(top: -200, left: 0, width: 200, height: 200):
    print("in x: positive, y:negative rect")
case Rect(top: 0, left: -200, width: 200, height: 200):
    print("in x: negative, y:positive rect")
case Rect(top: -200, left: -200, width: 200, height: 200):
    print("in x: negative, y:negative rect")
default:
    print("out of rect")
}


// 正規表現マッチング
struct Regexp {
    let pattern:String
}

func ~= (lhs:Regexp, rhs:String) -> Bool {
    return rhs.rangeOfString(lhs.pattern, options:.RegularExpressionSearch, range: nil, locale: nil) != nil
}

var str = "Hello, Swift"

switch str {
case Regexp(pattern: "Swift$"):
    print("Swift")
case Regexp(pattern: "Objective-C$"):
    print("Objective-C")
default:
    break
}